// file: qt_export.c
//
// started:
// 2003 March 4 dvb
//
// A command line utility for exporting quicktime movies
// into new formats
//

#include <stdio.h>
#include <CoreServices/CoreServices.h>
#include <QuickTime/QuickTime.h>

#include "qtc_utils.h"
#include "qtc_manpages.h"

#define k_string_size 1024



// |
// | The order for the parameters stacked up is like so:
// |
// | 1. Some defaults built into this program
// | 2. The settings loaded from a file
// | 3. The command line parameters
// | 4. The results of the user dialog
// |
// | each of these is optional, except the first one
// |



void man(void)
	{
	nr_print_version(__FILE__);
	#ifdef print_qt_export_man
		print_qt_export_man
	#else
		Uu
		U("No man page in this build.")
		Uu
	#endif

	exit(1);
	}

void version(void)
	{
	nr_print_version(__FILE__);
	exit(1);
	}

void usage(void)
	{
	nr_print_version(__FILE__);

	U("qt_export is a command line tool");
	U("for converting QuickTime movies to")
	U("different formats.");
	Uu
	U("usage: qt_export [options] <source movie file> <exported movie file>")
	Uu
	U("options:")
	Ue("--video=<compressor[,frame rate,quality,bits per pixel]>")
	Ue("--audio=<compressor[,samples per second,bits per sample,channel count]>")
	Ue("--datarate=<kilobytes per second>")
	Ue("--keyframerate=<frames per key frame>")
	Ue("--duration=<[start time,]end time>")
	Ue("--replacefile")
	Ue("--loadsettings=<export settings file>");
	Ue("--dodialog");
	Ue("--savesettings=<export settings file>");

    Ue("--sequencerate=<frames per second> treat source as image sequence");
	Uu
	U("also:")
	Ue("--video=0 to omit video")
	Ue("--audio=0 to omit audio")
	Uu
	U("also also:")
	Ue("\"qt_export --man | more\" for the full schpiel.")
	Uu
	U("by david van brink, poly@omino.com, subject line must begin with \"quicktime\"")
	Uu

	nr_printf(1,"");

	exit(0);
	}

// +-----------------------------
// |
// | settings from the --parameters
// |

typedef struct
	{
	// files (pointers to argv strings)

	char *source_movie_name;
	char *export_movie_name;

	// --loadsettings=, --dodialog, --savesettings=
	char *load_settings; // atom container as file
	int do_dialog;       // if so, do the QuickTime exporter dialog
	char *save_settings;

	// --exporter=subtype,mfr (default MooV,appl)
	OSType exporter_subtype;
	OSType exporter_mfr;


	// --video=compressor,frame rate,quality,bits per pixel
	OSType video_compressor;
	double frame_rate;
	int quality;  // arbitrary range 0 to 100.
	int pixel_size; // 1,2,4,8,16,32

	// --audio=compressor,samplerate,samplesize,channels
	OSType audio_compressor;
	double sample_rate;
	int sample_size; // 8 or 16, generally hey
	int channel_count; // 1 or 2

	double data_rate; // k per second
	int key_frame_rate; // frames per key frame

	// --duration=[start_time,]end_time
	double start_time; // seconds
	double end_time; // seconds, 0 means whole time

	int replace_file; // 1 means ok to delete existing file of same name
	int verbosity; // 0 silent, 1 normal, 2 verbose, 3 debuggery

	int exporter_audio; // 1 if were doing an audio-exporty-thing
	int exporter_video; // 1 if were doing a video-exporty-thing

    int b_supports_atoms;   // alas, apple 'MPEG'/0 mpeg 2 exporter does not
    int b_ignore_user_canceled; // special hack for 'MPEG'/0 mpeg2 exporter

    double sequence_rate;
	} s_parameter_settings;


// +---------------------
// | reverse engineered structures...

// | The "cdec" atom for QDM2 appears to be
// | sufficiently intertwined to be Not Worth reverse
// | engineering. You just have to do a --dodialog to
// | get repeatable settings for it.

// | Roxio's is really easy though, for MPG/Roxi
typedef struct
	{
	char format;       // 0: ntsc, 2: pal
	char shape;        // 1: fit, 2: crop, 3: fill
	char launch_toast; // 0: no, 1: yes
	char zero_1;       // must be 0
	char quality;      // 0: normal, 1: faster, 2: better
	char zeropad[11];  // must be 0
	} roxio_RtvS;

void r_print_parameter_settings(s_parameter_settings *ss)
	{
	nr_print_cr();
	nr_print_d_time_range("duration",ss->start_time,ss->end_time);
	if(ss->video_compressor != '0   ' || ss->exporter_video)
		{
		nr_print_ostype("video compressor",
				ss->video_compressor != '0   ' ? ss->video_compressor : ss->exporter_subtype);
		nr_print_d_frame_rate("video frames per second",ss->frame_rate);
		nr_print_int("video quality",ss->quality);
		nr_print_int("video bits per pixel",ss->pixel_size);
		nr_print_int("video key frame rate",ss->key_frame_rate);
		}
	else
		nr_print_info("video","off");

	nr_print_cr();

	if(ss->audio_compressor != '0   ' || ss->exporter_audio)
		{
		nr_print_ostype("audio compressor",
				ss->audio_compressor != '0   ' ? ss->audio_compressor : ss->exporter_subtype);
		nr_print_d_audio_format("audio format",ss->sample_rate,ss->sample_size,ss->channel_count);
		nr_print_cr();
		nr_print_d_data_rate("data kilobytes per second",ss->data_rate);
		nr_print_cr();
		nr_print_yesno("replace existing file",ss->replace_file);
		}
	else
		nr_print_info("audio","off");
	nr_print_cr();
	}

int r_atom_container_to_parameter_settings(QTAtomContainer ac,s_parameter_settings *ss)
	{
	SCSpatialSettings sps = {0};
	SCTemporalSettings sts = {0};
	SCDataRateSettings sdr = {0};
	int err = 0;
	char envi = 0;
	char enso = 0;
	UnsignedFixed uf_sample_rate;

	// |
	// | Video settings
	// |

	err = nr_get_deep_atom_data(ac,"videsptl",sizeof(sps),&sps);
	if(err == 0) // present?
		{
		ss->video_compressor = sps.codecType;
		ss->pixel_size = sps.depth;
		ss->quality = sps.spatialQuality * 100.0 / 1024.0;
		}

	err = nr_get_deep_atom_data(ac,"videtprl",sizeof(sts),&sts);
	if(err == 0) // present?
		{
		ss->frame_rate = f2d(sts.frameRate);
		ss->key_frame_rate = sts.keyFrameRate;
		}

	err = nr_get_deep_atom_data(ac,"videdrat",sizeof(sdr),&sdr);
	if(err == 0)
		{
		ss->data_rate = sdr.dataRate / 1024.0;
		}

	err = nr_get_deep_atom_data(ac,"envi",1,&envi);
	if(err || envi == 0)
		ss->video_compressor = '0   ';
	
	// |
	// | Audio Settings
	// |

	err = nr_get_deep_atom_data(ac,"sounssct",sizeof(OSType),&ss->audio_compressor);

	err = nr_get_deep_atom_data(ac,"sounssrt",sizeof(uf_sample_rate),&uf_sample_rate);
	if(err == 0)
		ss->sample_rate = f2d(uf_sample_rate);

	err = nr_get_deep_atom_data(ac,"sounssss",sizeof(ss->sample_size),&ss->sample_size);
	err = nr_get_deep_atom_data(ac,"sounsscc",sizeof(ss->channel_count),&ss->channel_count);

	err = nr_get_deep_atom_data(ac,"enso",1,&enso);
	if(err || enso == 0)
		ss->audio_compressor = '0   ';

	err = 0;

	return err;
	}

// +-----------------------------
// | Given an atom container, modify it
// | by any parameters in the parameter settings
// |
// | One subtlety: for the video compressor,
// | '0   ' means No Video! so delete the envi atom
// | NULL   means no opinion, leave those atoms alone
// | other  means that's the compressor we want.
// |
// | similarly for the audio compressor

int r_parameter_settings_atop_atom_container(s_parameter_settings *ss,QTAtomContainer ac)
	{
	SCSpatialSettings sps = {0};
	SCTemporalSettings sts = {0};
	SCDataRateSettings sdr = {0};
	int err = 0;

	// | Video settings, if video

	if(ss->video_compressor == '0   ')
		{
		// | no video!
		err = nr_delete_deep_atom(ac,"envi");  // presence of the atom makes video...
		bailerr(err,"could not delete envi atom");
		}
	else if(ss->video_compressor != 0) // other than null value...
		{
		sps.codecType = ss->video_compressor;
		sps.depth = ss->pixel_size;
		sps.spatialQuality = ss->quality * 1024 / 100; // quality goes 0x0100 to 0x0400

		sts.temporalQuality = 0x0200;
		sts.frameRate = d2f(ss->frame_rate);
		sts.keyFrameRate = ss->key_frame_rate;

		sdr.dataRate = (int)(ss->data_rate * 1024); // was k/second
		sdr.frameDuration = 0xffff8000; // I saw it in the defaults once
		sdr.minSpatialQuality = 0x0100; // I saw it in the defaults once
		sdr.minTemporalQuality = 0x0100; // I saw it in the defaults once

		err = nr_insert_deep_atom_byte(ac,"envi",1); // enable video
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_data(ac,"videsptl",sizeof(sps),&sps);
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_data(ac,"videtprl",sizeof(sts),&sts);
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_data(ac,"videdrat",sizeof(sdr),&sdr);
		bailerr(err,"could not create settings atoms");

		long mfr = 0; // TRY IT! 2006.03.07
		err = nr_insert_deep_atom_data(ac,"videcmfr",sizeof(mfr),&mfr);


		}

	if(ss->audio_compressor == '0   ')
		{
		err = nr_delete_deep_atom(ac,"enso"); // disable audio (sound)
		bailerr(err,"could not delete enso atom");
		}
	else if(ss->audio_compressor != 0)
		{
		err = nr_insert_deep_atom_byte(ac,"enso",1); // enable audio (sound)
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_long(ac,"sounssct",ss->audio_compressor);
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_long(ac,"sounssrt",d2f(ss->sample_rate));
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_short(ac,"sounssss",ss->sample_size);
		bailerr(err,"could not create settings atoms");
		err = nr_insert_deep_atom_short(ac,"sounsscc",ss->channel_count);
		bailerr(err,"could not create settings atoms");
		}

go_home:
	nr_printf(3,"r_parameter_settings_atop_atom_container: returning %d bytes",
			ac ? GetHandleSize((Handle)ac) : 0);

	return err;
	}

// +-------------------------------
// | applying parameter settings... we do it all, here.
// | 


OSErr r_progress_proc
		(
		Movie theMovie,
		short message,
		short whatOperation,
		Fixed percentDone,
		long refcon
		)
	{
	s_parameter_settings *ss = (s_parameter_settings *)refcon;
	double percent = percentDone * 100.0 / 65536.0;
	static double recent_percent = -100;

	// |
	// | dont show more than uhhh 200 progress notes
	// | also, avoid spewing a zillion 100% progresses at
	// | the end of Roxio's stupid mpeg exporter
	// | (which I really like for the most part -- dvb)
	// |

	if(percent - recent_percent < 0.5)
		goto go_home;
	recent_percent = percent;

	nr_printf(1,"# progress: (%03d:%03d) %6.2f%%       %s --> %s\n",
			(int)whatOperation,
			(int)message,
			percent,
			ss->source_movie_name,
			ss->export_movie_name
			);

	// could show output file size here, wouldnt that be kinda cool

go_home:
	return 0;
	}


int r_args_to_settings(int argc,char **argv,s_parameter_settings *ssOut)
{
	s_parameter_settings ss = {0};
	int found_it = 0;
	int err = 0;

    ss.sequence_rate = -1;
    ss.b_supports_atoms = 1;
    ss.b_ignore_user_canceled = 0;

  	// |
	// | needs two movie arguments to work... or none
	// | (for just settings-stuffs)
	// |

	found_it = nr_find_arg(argc,argv,"1",0,&ss.source_movie_name);
	found_it = nr_find_arg(argc,argv,"2",0,&ss.export_movie_name);

	if(ss.source_movie_name && !ss.export_movie_name)
		{
		err = -1;
		obailerr(err,"If you provide a source file then you must provide an export file too");
		}

	// | set the output verbosity early

	ss.verbosity = nr_find_arg_int(argc,argv,"verbosity",0,1);
	g_verbosity = ss.verbosity; // global

	// |
	// | populate the parameter settings
	// |

	// | these first parameter settings come ONLY from command line
	// | (never from stored atoms)

	nr_find_arg(argc,argv,"loadsettings",0,&ss.load_settings);
	nr_find_arg(argc,argv,"savesettings",0,&ss.save_settings);
	nr_find_arg(argc,argv,"dodialog",&ss.do_dialog,0);
	ss.replace_file = nr_find_arg_int(argc,argv,"replacefile",0,0);
    ss.sequence_rate = nr_find_arg_double(argc,argv,"sequencerate",0,ss.sequence_rate);

	// | populate parameter settings with defaults

	ss.exporter_subtype = 'MooV';
	ss.exporter_mfr = 'appl';

	// |
	// | !!! TODO (as they say): fill in an exporter type based on the
	// | filename extension of the export file. That way, qt_export blah.mp3 foo.aiff
	// | would just do what you meant. For example.
	// |

	ss.video_compressor = 'SVQ3';   // sorenson...
	ss.frame_rate = 0.0;
	ss.quality = 50;
	ss.pixel_size = 32;

	ss.audio_compressor = 'QDM2';
	ss.audio_compressor = 'twos';  // turns out that QDM2 is a bad choice for a default!
	ss.sample_rate = 44100;
	ss.sample_size = 16;
	ss.channel_count = 2;

	ss.exporter_subtype = nr_find_arg_ostype(argc,argv,"exporter",0,ss.exporter_subtype);

	// | User changing exporter? we set up some reasonable defaults here for mfr
	// | (if mfr is 'appl', its just a default, we might change it)
	if(ss.exporter_mfr == 'appl')
		switch(ss.exporter_subtype)
			{
			case 'AIFF':
			case 'sfil': // system 7 sound
			case 'snd ': // classic MacOS sound file
				ss.exporter_mfr = 'soun';
				ss.audio_compressor = 'twos';
				ss.video_compressor = 0;
				ss.exporter_audio = 1;
				break;

			case 'WAVE':
				ss.exporter_mfr = 'soun';
				ss.audio_compressor = 'raw ';
				ss.video_compressor = 0;
				ss.exporter_audio = 1;
				break;

			case 'MPG ':  // default mpeg1 exporter is roxio's
				ss.exporter_mfr = 'Roxi';
				ss.exporter_audio = 1;
				ss.exporter_video = 1;
				break;

			case 'MPEG': // mpeg 2
				ss.exporter_mfr = 0;  // strangely, mfr is 0
				ss.exporter_audio = 1;
				ss.exporter_video = 1;
				break;
			}

	ss.exporter_mfr = nr_find_arg_ostype(argc,argv,"exporter",1,ss.exporter_mfr);

	ss.data_rate = 100;    // kilobytes per second, default (video)
	ss.key_frame_rate = 60;

	ss.start_time = 0.0;
	ss.end_time = 0.0;

	ss.video_compressor = nr_find_arg_ostype(argc,argv,"video",0,ss.video_compressor);
	ss.frame_rate = nr_find_arg_double(argc,argv,"video",1, ss.frame_rate);
	ss.quality = nr_find_arg_double(argc,argv,"video",2,ss.quality);
	ss.pixel_size = nr_find_arg_int(argc,argv,"video",3,ss.pixel_size);

	ss.audio_compressor = nr_find_arg_ostype(argc,argv,"audio",0,ss.audio_compressor);
	ss.sample_rate = nr_find_arg_double(argc,argv,"audio",1,ss.sample_rate);
	ss.sample_size = nr_find_arg_int(argc,argv,"audio",2,ss.sample_size);
	ss.channel_count = nr_find_arg_int(argc,argv,"audio",3,ss.channel_count);

	// | going backwards, let some audio/video compressors override
	// | the exporter choice. 

	switch(ss.audio_compressor)
		{
		case 'aiff':
		case 'AIFF':
			ss.exporter_subtype = 'AIFF';
			ss.exporter_mfr = 'soun';
			ss.audio_compressor = 'twos';
		common_tail_1:
			ss.video_compressor = 0;
			nr_printf(3,"# (setting exporter to %s,%s, fmt to %s)\n",
					o2c(ss.exporter_subtype),
					o2c(ss.exporter_mfr),
					o2c(ss.audio_compressor));
			ss.exporter_audio = 1;
			break;

		case 'wav ':
		case 'WAV ':
		case 'wave':
		case 'WAVE':
			ss.exporter_subtype = 'WAVE';
			ss.exporter_mfr = 'soun';
			ss.audio_compressor = 'raw ';
			goto common_tail_1;
		}
	switch(ss.video_compressor)
		{
		case 'mpg2':
		case 'MPG2':
		case 'mpeg':
		case 'MPEG':
			ss.exporter_subtype = 'MPEG';
			ss.exporter_mfr = 0;
			ss.exporter_audio = 1;
			ss.exporter_video = 1;
			break;

		case 'mpg1':
		case 'MPG1':
		case 'mpg ':
		case 'MPG ':
			ss.exporter_subtype = 'MPG ';
			ss.exporter_mfr = 'Roxi';
			ss.exporter_audio = 1;
			ss.exporter_video = 1;
			break;
		}

	ss.data_rate = nr_find_arg_double(argc,argv,"datarate",0,ss.data_rate);
	ss.key_frame_rate = nr_find_arg_int(argc,argv,"keyframerate",0,ss.key_frame_rate);

	ss.start_time = nr_find_arg_double(argc,argv,"duration",0,ss.start_time);
	ss.end_time = nr_find_arg_double(argc,argv,"duration",1,ss.end_time);

	if(ss.end_time == 0)
		{
		ss.end_time = ss.start_time;
		ss.start_time = 0;
		}

go_home:
	if(!err)
		*ssOut = ss;
	return err;
}



int doMovieSequenceImport(ComponentInstance ci,Movie mo,s_parameter_settings *ss)
{
	int err = 0;
	// analyze filename of source to discern the numbering pattern of the files...
	
	sequence_stuff seqstuff = {0};
	nr_filename_to_sequence_stuff(ss->source_movie_name,&seqstuff);
	if(seqstuff.digitCount < 1)
		bail(1,nr_sprintf("filename \"%d\" is unnumbered, cant import sequence",ss->source_movie_name));

printf("HEY! TODO ok, import up to %d frames at %f fps\n",seqstuff.indexMax,ss->sequence_rate);
	
go_home:
    return err;
}


int doMovieExport(ComponentInstance ci,Movie mo,s_parameter_settings *ss)
{
	int err = 0;
	if(mo)
		{
		FSSpec fs;
		Boolean valid;
		TimeValue start_time;
		TimeValue duration;
		TimeScale movie_time_scale;

		// |
		// | figure offset & duration, with some funky
		// |

		movie_time_scale = GetMovieTimeScale(mo);
		start_time = (int)(ss->start_time * movie_time_scale);
		duration = (int)((ss->end_time - ss->start_time) * movie_time_scale);

		if(duration == 0)
			{
			start_time = 0;
			duration = GetMovieDuration(mo);
			}

		// |
		// | Some abort conditions...
		// |

		if(ss->video_compressor == 'QDM2'
				&& !ss->do_dialog
				&& !ss->load_settings)
			{
			err = -1;
			obailerr(err,"QDM2 audio requires a settings file or setup dialog");
			}

		// |
		// | See about blasting the existing file
		// |

		err = NativePathNameToFSSpec(ss->export_movie_name,&fs,0);
		if(err == fnfErr) err = 0; // mixed doc on whether you get fnfErr with this call!
		obailerr(err,"NativePathNameToFSSpec");

		err = nr_file_exists(ss->export_movie_name);
		if(err == 0) // file already exists!
			{
			if(ss->replace_file)
				{
				err = FSpDelete(&fs);
				obailerr(err,nr_sprintf("could not delete existing file %s",ss->export_movie_name));
				}
			else
				{
				err = -1;
				obailerr(err, nr_sprintf("file %s already exists, consider --replacefile",
						ss->export_movie_name));
				}
			}
		else if(err != fnfErr) // we hope for fnfErr
			{
			obailerr(err,nr_sprintf("file error %d",err));
			}

		err = MovieExportValidate(ci,mo,0,&valid);
		nr_printf(3,"movie validate: %d / %d\n",err,valid);
		if(!valid & !err)
			err = -1;
		obailerr(err,"invalid settings");

		// |
		// | Create the receiving file
		// |

		err = FSpCreate(&fs,'TVOD',0,smSystemScript);
		obailerr(err,"Could not create new file");
		
		// |
		// | And at last, export the new movie (or other
		// | file type)
		// |

		nr_printf(3,"%d ci=: 0x%08x\n",__LINE__,ci);
		err = MovieExportToFile(ci,&fs,mo,0,start_time,duration);
		obailerr(err,"MovieExportToFile failed");
		}
go_home:
	return err;
}















//int main(int argc,char *argv[])
main(int argc,char **argv)
	{
	int i;
	OSErr err = 0;
	ComponentInstance ci;
	int found_it = 0;
	s_parameter_settings ss = {0};
	QTAtomContainer ac = 0;

	setbuf(stdout,0);

	if(argc == 1)
		usage();

	found_it = nr_find_arg(argc,argv,"help",0,0);
	if(found_it)
		usage();

	if(nr_find_arg(argc,argv,"man",0,0))
		man();
		
	if(nr_find_arg(argc,argv,"version",0,0))
		version();

	nr_printf(1,"");

    // |
    // | these fields are only icky hacks for MPEG 2 exporter
    // |
	
	r_args_to_settings(argc,argv,&ss);

	// +---------------------------
	// | all done reading argc and argv. no more allowed.
	// |
	argc = 0;
	argv = 0;

	nr_enter_movies();
	
	// |
	// | Open up the movie (if any)
	// |

	Movie mo = 0;
	if(ss.source_movie_name)
		{
		mo = nr_new_movie_from_file(ss.source_movie_name);
		obailnil(mo,nr_sprintf("could not open movie %s",ss.source_movie_name));
		}




	ci = nr_open_component( 'spit',ss.exporter_subtype,ss.exporter_mfr);
	nr_printf(3,"ci is 0x%08x\n",ci);
	obailnil(ci,
			nr_sprintf("could not open export component %s:%s (%08x:%08x)",
					o2c(ss.exporter_subtype),
					o2c(ss.exporter_mfr),
					ss.exporter_subtype,
					ss.exporter_mfr));


	// +------------------------------
	// | preload some exporters with specific data
    // | or modality
	// |
	if(ss.exporter_subtype == 'MPG '
			&& ss.exporter_mfr == 'Roxi')
		{
		QTAtomContainer roxi_ac;
		roxio_RtvS rtvs = {0};

		rtvs.format = 0; // ntsc
		rtvs.shape = 1; // fit
		rtvs.launch_toast = 0; // its already zero, but I feel so strongly
		rtvs.quality = 2; // better

		err = QTNewAtomContainer(&roxi_ac);
		obailerr(err,"Could not create atom container");

		// must be atom id 1000
		err = nr_insert_deep_atom_data_id(roxi_ac,"RtvS",sizeof(rtvs),&rtvs,1000);
		obailerr(err,"Could not insert RtvS atom for Roxio MPEG");

		err = MovieExportSetSettingsFromAtomContainer(ci,roxi_ac);
		obailerr(err,"Could not set settings for Roxio MPEG");

		QTDisposeAtomContainer(roxi_ac); // polite, not that it matters
		}
    else if (ss.exporter_subtype == 'MPEG'
            && ss.exporter_mfr == 0)
        {
        // |
        // | The apple MPEG2 exporter -- as provided with DVDStudioPro --
        // | is pretty obnoxious. It does *not* support QTAtom storage
        // | of parameters, and, additionally, it seems to have a bug
        // | where after the user dialog it always says, "ha! the
        // | user canceled!"
        // |
        //ss.b_supports_atoms = 0;
        ss.b_ignore_user_canceled = 1;
        }

	// |
	// | done with any special preloadery
	// +-------------------------------------

    // | various atomic things

    if(ss.b_supports_atoms)
        {

        // |
        // | loading from existing settings? do so then.
        // |

        if(ss.load_settings)
            {
            err = nr_file_to_atom_container(ss.load_settings,&ac);
            obailerr(err,nr_sprintf("could not load settings file %s",ss.load_settings));

            err = r_atom_container_to_parameter_settings(ac,&ss);
            obailerr(err,"could not extract settings from settings file atom");
            }
        else
            {
            // |
            // | otherwise, get a starter-atom from the exporter component
            // |

            err = MovieExportGetSettingsAsAtomContainer(ci,&ac);
            obailerr(err,"Initial MovieExportGetSettingsAsAtomContainer");
            }

        // |
        // | Convert settings to Atom, and possibly invoke the
        // | the export dialog
        // |

        // | IF you are using load_settings or do_dialog, 
        // | THEN we make no attempt to use the command line switches.
        // | We trust completely the dialog or stashed settings.
        // |

        if((!ss.load_settings) && (!ss.do_dialog))  // this is the test
            {
            err = r_parameter_settings_atop_atom_container(&ss,ac);
            obailerr(err,"r_parameter_settings_atop_atom_container");
            }

        err = MovieExportSetSettingsFromAtomContainer(ci,ac);
        obailerr(err,"MovieExportSetSettingsFromAtomContainer");
        } // b_supports_atoms

	if(ss.do_dialog)
		{
		Boolean canceled;

		nr_printf(1,"\n# (Settings dialog may be hidden by this terminal window...)\n\n");

            {
            // thank you sef!
            // sean writes:
            // Courtesy of Deric Horn and myself.
            // 2004.05.28
            ProcessSerialNumber psn;

            err = GetCurrentProcess(&psn);
            if (err == noErr)
                {
                (void)SetFrontProcess(&psn);
                }
            }

		TimeValue movie_duration = mo ? GetMovieDuration(mo) : 0;
		err = MovieExportDoUserDialog(ci,mo,nil,0,movie_duration,&canceled);
		bailerr(err,"MovieExportDoUserDialog");

		if(canceled)
			{
            if(ss.b_ignore_user_canceled)
                printf("# (ignoring MovieExportDoUserDialog canceled)\n");
            else
                {
                err = -1;
                obailerr(err,"user canceled");
			    }
			}
		}

	// | get the atom container from the export component, in case
	// | the exporter changed it trickily...

    if(ss.b_supports_atoms)
        {
        QTDisposeAtomContainer(ac);
        err = MovieExportGetSettingsAsAtomContainer(ci,&ac);
        obailerr(err,"MovieExportGetSettingsAsAtomContainer");

        // | one last conversion back to ss, for printing

        r_atom_container_to_parameter_settings(ac,&ss);

        // | and maybe save those settings.

        if(ss.save_settings)
            {
            err = nr_atom_container_to_file(ac,ss.save_settings);
            obailerr(err,nr_sprintf("could not save settings file %s",ss.save_settings));
            }

        QTDisposeAtomContainer(ac);
        ac = 0;
        } // b_supports_atoms

	r_print_parameter_settings(&ss);

	err = MovieExportSetProgressProc(ci,r_progress_proc,(long)&ss);
	//obailerr(err,"MovieExportSetProgressProc");
    if(err < 0)
        {
	    nr_printf(1,"# (MovieExportSetProgressProc: %d)",err);
        err = 0;
        }
	
	// |
	// | If we are actually exporting, do it now
	// |
	
    if(ss.sequence_rate >= 0)
        doMovieSequenceImport(ci,mo,&ss);
    else
	    doMovieExport(ci,mo,&ss);
	

go_home:
	if(err)
		fprintf(stderr,"### error is %d\n",err);

	if(ci)
		CloseComponent(ci);
	if(mo)
		DisposeMovie(mo);

	nr_printf(1,"");
	if(err)
		exit(1);

	exit(0);
	}

// end of file
